package com.reactnativestripesdk

import com.facebook.react.bridge.ReadableArray
import com.facebook.react.bridge.ReadableMap
import com.facebook.react.common.MapBuilder
import com.facebook.react.uimanager.SimpleViewManager
import com.facebook.react.uimanager.ThemedReactContext
import com.facebook.react.uimanager.annotations.ReactProp
import com.stripe.android.model.PaymentMethodCreateParams

const val CARD_FIELD_INSTANCE_NAME = "CardFieldInstance"

class StripeSdkCardViewManager : SimpleViewManager<StripeSdkCardView>() {
  override fun getName() = "CardField"

  private var cardViewInstanceMap: MutableMap<String, Any?> = mutableMapOf()

  override fun getExportedCustomDirectEventTypeConstants(): MutableMap<String, Any> {
    return MapBuilder.of(
      CardFocusEvent.EVENT_NAME, MapBuilder.of("registrationName", "onFocusChange"),
      CardChangedEvent.EVENT_NAME, MapBuilder.of("registrationName", "onCardChange"))
  }

  override fun receiveCommand(root: StripeSdkCardView, commandId: String?, args: ReadableArray?) {
    when (commandId) {
      "focus" -> root.requestFocusFromJS()
      "blur" -> root.requestBlurFromJS()
      "clear" -> root.requestClearFromJS()
    }
  }

  @ReactProp(name = "dangerouslyGetFullCardDetails")
  fun setDangerouslyGetFullCardDetails(view: StripeSdkCardView, dangerouslyGetFullCardDetails: Boolean = false) {
    view.setDangerouslyGetFullCardDetails(dangerouslyGetFullCardDetails);
  }

  @ReactProp(name = "postalCodeEnabled")
  fun setPostalCodeEnabled(view: StripeSdkCardView, postalCodeEnabled: Boolean = true) {
    view.setPostalCodeEnabled(postalCodeEnabled);
  }

  @ReactProp(name = "autofocus")
  fun setAutofocus(view: StripeSdkCardView, autofocus: Boolean = false) {
    view.setAutofocus(autofocus);
  }

  @ReactProp(name = "cardStyle")
  fun setCardStyle(view: StripeSdkCardView, cardStyle: ReadableMap) {
    view.setCardStyle(cardStyle);
  }

  @ReactProp(name = "placeholder")
  fun setPlaceHolders(view: StripeSdkCardView, placeholder: ReadableMap) {
    view.setPlaceHolders(placeholder);
  }

  override fun createViewInstance(reactContext: ThemedReactContext): StripeSdkCardView {
    // as it's reasonable we handle only one CardField component on the same screen
    // TODO: temporary commented out due to android state persistence and improper behavior after app reload
//    if (cardViewInstanceMap[CARD_FIELD_INSTANCE_NAME] != null) {
//      val exceptionManager = reactContext.getNativeModule(ExceptionsManagerModule::class.java)
//      val error: WritableMap = WritableNativeMap()
//      error.putString("message", "Only one CardField component on the same screen allowed")
//      exceptionManager?.reportException(error)
//    }

    cardViewInstanceMap[CARD_FIELD_INSTANCE_NAME] = StripeSdkCardView(reactContext)
    return cardViewInstanceMap[CARD_FIELD_INSTANCE_NAME] as StripeSdkCardView
  }

  override fun onDropViewInstance(view: StripeSdkCardView) {
    super.onDropViewInstance(view)

    this.cardViewInstanceMap[CARD_FIELD_INSTANCE_NAME] = null
  }

  fun getCardViewInstance(): StripeSdkCardView? {
    if (cardViewInstanceMap[CARD_FIELD_INSTANCE_NAME] != null) {
      return cardViewInstanceMap[CARD_FIELD_INSTANCE_NAME] as StripeSdkCardView
    }
    return null
  }

  fun setCardDetails(value: ReadableMap, reactContext: ThemedReactContext) {
    val number = getValOr(value, "number", null)
    val expirationYear = getIntOrNull(value, "expirationYear")
    val expirationMonth = getIntOrNull(value, "expirationMonth")
    val cvc = getValOr(value, "cvc", null)

    val cardViewInstance = getCardViewInstance() ?: createViewInstance(reactContext)
    cardViewInstance.cardParams = PaymentMethodCreateParams.Card.Builder()
            .setNumber(number)
            .setCvc(cvc)
            .setExpiryMonth(expirationMonth)
            .setExpiryYear(expirationYear)
            .build()
  }
}
